#![allow(non_camel_case_types)]
#![allow(dead_code)]
// ***********************************************************************
//  Module:    cpic.rs                                                   *
//                                                                       *
//  Purpose:   Defines the CPI-C API parameters and routines for Rust to *
//             to programs to make APPC transactions.                    *
//                                                                       *
// To compile this with a program, install in cargo src directory and    *
// include this module using the mod statement in Rust:                  *
//                                                                       *
// mod cpic;                                                             *
//                                                                       *
// ***********************************************************************
//                                                                       *
// � COPYRIGHT IBM Corporation 1989-2022                                 *
//                                                                       *
// ***********************************************************************

use std::os::raw::{c_int, c_char};

//  Constant Definitions    

// Values for the conversation_state 

  pub const CM_INITIALIZE_STATE:              i32 =  2;
  pub const CM_SEND_STATE:                    i32 =  3;
  pub const CM_RECEIVE_STATE:                 i32 =  4;
  pub const CM_SEND_PENDING_STATE:            i32 =  5;
  pub const CM_CONFIRM_STATE:                 i32 =  6;
  pub const CM_CONFIRM_SEND_STATE:            i32 =  7;
  pub const CM_CONFIRM_DEALLOCATE_STATE:      i32 =  8;
  pub const CM_DEFER_RECEIVE_STATE:           i32 =  9;
  pub const CM_DEFER_DEALLOCATE_STATE:        i32 =  10;
  pub const CM_SYNC_POINT_STATE:              i32 =  11;
  pub const CM_SYNC_POINT_SEND_STATE:         i32 =  12;
  pub const CM_SYNC_POINT_DEALLOCATE_STATE:   i32 =  13;
  pub const CM_INITIALIZE_INCOMING_STATE:     i32 =  14;
  pub const CM_SEND_ONLY_STATE:               i32 =  15;
  pub const CM_RECEIVE_ONLY_STATE:            i32 =  16;
  pub const CM_SEND_RECEIVE_STATE:            i32 =  17;
  pub const CM_PREPARED_STATE:                i32 =  18;


// Values for the conversation_type  

  pub const CM_BASIC_CONVERSATION:   i32 =  0;
  pub const CM_MAPPED_CONVERSATION:  i32 =  1;


// Values for data_received     

  pub const CM_NO_DATA_RECEIVED:          i32 =  0;
  pub const CM_DATA_RECEIVED:             i32 =  1;
  pub const CM_COMPLETE_DATA_RECEIVED:    i32 =  2;
  pub const CM_INCOMPLETE_DATA_RECEIVED:  i32 =  3;


// Values for the deallocate_type 0

  pub const CM_DEALLOCATE_SYNC_LEVEL:  i32 =  0;
  pub const CM_DEALLOCATE_FLUSH:       i32 =  1;
  pub const CM_DEALLOCATE_CONFIRM:     i32 =  2;
  pub const CM_DEALLOCATE_ABEND:       i32 =  3;


// Values for the error_direction 
 
  pub const CM_RECEIVE_ERROR:  i32 =  0;
  pub const CM_SEND_ERROR:     i32 =  1;


// Values for fill type 

  pub const CM_FILL_LL:      i32 =  0;
  pub const CM_FILL_BUFFER:  i32 =  1;


// Values for prepare_to_receive_type 
  
  pub const CM_PREP_TO_RECEIVE_SYNC_LEVEL:  i32 =  0;
  pub const CM_PREP_TO_RECEIVE_FLUSH:       i32 =  1;
  pub const CM_PREP_TO_RECEIVE_CONFIRM:     i32 =  2;


// Values for the processing_mode   

  pub const CM_BLOCKING:     i32 = 0;
  pub const CM_NON_BLOCKING: i32 = 1;


// Values for the receive_type   

  pub const CM_RECEIVE_AND_WAIT:   i32 =  0;
  pub const CM_RECEIVE_IMMEDIATE:  i32 =  1;


// Values for request_to_send_received   
 
  pub const CM_REQ_TO_SEND_NOT_RECEIVED:  i32 =  0;
  pub const CM_REQ_TO_SEND_RECEIVED:      i32 =  1;

// Values for signal_behavior_type  

  pub const XC_SIGNAL_BEHAVIOR_NO_RETRY:       i32 =  0;
  pub const XC_SIGNAL_BEHAVIOR_INFINITE_RETRY: i32 =  1;

// Values for return_code 

  pub const CM_OK:                            i32 =  0;
  pub const CM_ALLOCATE_FAILURE_NO_RETRY:     i32 =  1;
  pub const CM_ALLOCATE_FAILURE_RETRY:        i32 =  2;
  pub const CM_CONVERSATION_TYPE_MISMATCH:    i32 =  3;
  pub const CM_PIP_NOT_SPECIFIED_CORRECTLY:   i32 =  5;
  pub const CM_SECURITY_NOT_VALID:            i32 =  6;
  pub const CM_SYNC_LVL_NOT_SUPPORTED_LU:     i32 =  7;
  pub const CM_SYNC_LEVEL_NOT_SUPPORTED_LU:   i32 =  7;
  pub const CM_SYNC_LVL_NOT_SUPPORTED_PGM :   i32 =  8;
  pub const CM_SYNC_LEVEL_NOT_SUPPORTED_PGM:  i32 =  8;
  pub const CM_TPN_NOT_RECOGNIZED:            i32 =  9;
  pub const CM_TPN_NOT_RECOGNISED:            i32 =  9;
  pub const CM_TP_NOT_AVAILABLE_NO_RETRY:     i32 =  10;
  pub const CM_TP_NOT_AVAILABLE_RETRY:        i32 =  11;
  pub const CM_DEALLOCATED_ABEND:             i32 =  17;
  pub const CM_DEALLOCATED_NORMAL:            i32 =  18;
  pub const CM_PARAMETER_ERROR:               i32 =  19;
  pub const CM_PRODUCT_SPECIFIC_ERROR:        i32 =  20;
  pub const CM_PROGRAM_ERROR_NO_TRUNC:        i32 =  21;
  pub const CM_PROGRAM_ERROR_PURGING:         i32 =  22;
  pub const CM_PROGRAM_ERROR_TRUNC:           i32 =  23;
  pub const CM_PROGRAM_PARAMETER_CHECK:       i32 =  24;
  pub const CM_PROGRAM_STATE_CHECK:           i32 =  25;
  pub const CM_RESOURCE_FAILURE_NO_RETRY:     i32 =  26;
  pub const CM_RESOURCE_FAILURE_RETRY:        i32 =  27;
  pub const CM_UNSUCCESSFUL:                  i32 =  28;
  pub const CM_DEALLOCATED_ABEND_SVC:         i32 =  30;
  pub const CM_DEALLOCATED_ABEND_TIMER:       i32 =  31;
  pub const CM_SVC_ERROR_NO_TRUNC:            i32 =  32;
  pub const CM_SVC_ERROR_PURGING:             i32 =  33;
  pub const CM_SVC_ERROR_TRUNC:               i32 =  34;
  pub const CM_OPERATION_INCOMPLETE:          i32 =  35;
  pub const CM_SYSTEM_EVENT:                  i32 =  36;
  pub const CM_OPERATION_NOT_ACCEPTED:        i32 =  37;
  pub const CM_CONVERSATION_ENDING:           i32 =  38;
  pub const CM_SEND_RCV_MODE_NOT_SUPPORTED:   i32 =  39;
  pub const CM_BUFFER_TOO_SMALL:              i32 =  40;
  pub const CM_EXP_DATA_NOT_SUPPORTED:        i32 =  41;
  pub const CM_DEALLOC_CONFIRM_REJECT:        i32 =  42;
  pub const CM_ALLOCATION_ERROR:              i32 =  43;
  pub const CM_RETRY_LIMIT_EXCEEDED:          i32 =  44;
  pub const CM_NO_SECONDARY_INFORMATION:      i32 =  45;
  pub const CM_SECURITY_NOT_SUPPORTED:        i32 =  46;
  pub const CM_SECURITY_MUTUAL_FAILED:        i32 =  47;
  pub const CM_CALL_NOT_SUPPORTED:            i32 =  48;
  pub const CM_PARM_VALUE_NOT_SUPPORTED:      i32 =  49;
  pub const CM_TAKE_BACKOUT:                  i32 =  100;
  pub const CM_DEALLOCATED_ABEND_BO:          i32 =  130;
  pub const CM_DEALLOCATED_ABEND_SVC_BO:      i32 =  131;
  pub const CM_DEALLOCATED_ABEND_TIMER_BO:    i32 =  132;
  pub const CM_RESOURCE_FAIL_NO_RETRY_BO:     i32 =  133;
  pub const CM_RESOURCE_FAILURE_RETRY_BO:     i32 =  134;
  pub const CM_DEALLOCATED_NORMAL_BO:         i32 =  135;
  pub const CM_CONV_DEALLOC_AFTER_SYNCPT:     i32 =  136;
  pub const CM_INCLUDE_PARTNER_REJECT_BO:     i32 =  137;


// Values for return_control                        

  pub const CM_WHEN_SESSION_ALLOCATED:  i32 =  0;
  pub const CM_IMMEDIATE:               i32 =  1;


// Values for the send_type   
 
  pub const CM_BUFFER_DATA:               i32 =  0;
  pub const CM_SEND_AND_FLUSH:            i32 =  1;
  pub const CM_SEND_AND_CONFIRM:          i32 =  2;
  pub const CM_SEND_AND_PREP_TO_RECEIVE:  i32 =  3;
  pub const CM_SEND_AND_DEALLOCATE:       i32 =  4;


// Values for status_received 

  pub const CM_NO_STATUS_RECEIVED:            i32 =  0;
  pub const CM_SEND_RECEIVED:                 i32 =  1;
  pub const CM_CONFIRM_RECEIVED:              i32 =  2;
  pub const CM_CONFIRM_SEND_RECEIVED:         i32 =  3;
  pub const CM_CONFIRM_DEALLOC_RECEIVED:      i32 =  4;
  pub const CM_TAKE_COMMIT:                   i32 =  5;
  pub const CM_TAKE_COMMIT_SEND:              i32 =  6;
  pub const CM_TAKE_COMMIT_DEALLOCATE:        i32 =  7;
  pub const CM_TAKE_COMMIT_DATA_OK:           i32 =  8;
  pub const CM_TAKE_COMMIT_SEND_DATA_OK:      i32 =  9;
  pub const CM_TAKE_COMMIT_DEALLOC_DATA_OK:   i32 =  10;
  pub const CM_PREPARE_OK:                    i32 =  11;
  pub const CM_JOINT_TRANSACTION:             i32 =  12;

// Values for the sync_level            

  pub const CM_NONE:                     i32 =  0;
  pub const CM_CONFIRM:                  i32 =  1;
  pub const CM_SYNC_POINT:               i32 =  2;
  pub const CM_SYNC_POINT_NO_CONFIRM:    i32 =  3;


// Values for end type (DCL addition)   

  pub const CM_HARD:    i32 = 0;
  pub const CM_SOFT:    i32 = 1;
  

// Values for maximum sizes of strings and buffers 

  pub const CM_CID_SIZE:   i32 = 8;    // conversation ID
  pub const CM_CTX_SIZE:   i32 = 32;   // context ID     
  pub const CM_LD_SIZE:    i32 = 512;  // log data       
  pub const CM_MN_SIZE:    i32 = 8;    // mode name        
  pub const CM_PLN_SIZE:   i32 = 17;   // partner LU name  
  pub const CM_PW_SIZE:    i32 = 10;   // password         
  pub const CM_SDN_SIZE:   i32 = 8;    // symbolic destination name
  pub const CM_TPN_SIZE:   i32 = 64;   // TP name         
  pub const CM_UID_SIZE:   i32 = 10;   // userid ID         

// API functions

#[link(name = "cpic")]
#[link(name = "appc")]
#[link(name = "nof")]
#[link(name = "sna_r")]
#[link(name = "pthread")]
extern {

    pub fn cminit( conversation_id: *mut c_char,
                   sym_dest_name:  *const c_char,
                   return_code:    *mut c_int);

    pub fn cmepln( conversation_id: *const c_char,
                   partner_LU_name: *mut c_char,
                   partner_LU_name_length: *mut c_int,
                   return_code:    *mut c_int);

    pub fn cmssl( conversation_id: *const c_char,
                  sync_level: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmstpn( conversation_id: *const c_char,
                   TP_name: *mut c_char,
                   TP_name_length: c_int,
                   return_code:    *mut c_int);    

    pub fn cmemn( conversation_id: *const c_char,
                  mode_name: *mut c_char,
                  mode_name_length: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmallc( conversation_id: *const c_char,
                    return_code:    *mut c_int);

    pub fn cmsend( conversation_id: *const c_char,
                   buffer: *const c_char,
                   buffer_length: *const c_int,
                   request_to_send_received: *mut c_int,
                   return_code:    *mut c_int);

    pub fn cmrcv( conversation_id: *const c_char,
                  buffer: *const c_char,
                  requested_length: *const c_int,
                  data_received: *mut c_int,
                  received_length: *mut c_int,
                  status_received: *mut c_int,
                  request_to_send_received: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmsdt( conversation_id: *const c_char,
                  deallocate_type: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmdeal( conversation_id: *const c_char,
                   return_code:    *mut c_int);

    pub fn cmcnvi(string:      *const c_char,
                  str_len_ptr: *const c_int,
                  return_code: *mut c_int);
                                            
    pub fn cmcnvo(string:      *const c_char,
                  str_len_ptr: *const c_int,
                  return_code: *mut c_int);

    pub fn cmaccp(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmacci(conversation_id: *const c_char,
                return_code:    *mut c_int);

    pub fn cmcanc(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmcfm(conversation_id: *const c_char,
                 request_to_send_received: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmcfmd(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmwait(conversation_id: *const c_char,
                  converstation_return_code: *const c_int,
                   return_code:    *mut c_int);

    pub fn cmchck(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmectx(conversation_id: *const c_char,
                  context_id: *mut c_char,
                  context_id_lenght: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmsctx(conversation_id: *const c_char,
                  converstation_type: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmecs(conversation_id: *const c_char,
                 conversation_state: *mut c_int,
                 return_code:    *mut c_int);

    pub fn cmect(conversation_id: *const c_char,
                 conversation_type: *mut c_int,
                 return_code:    *mut c_int);
    
    pub fn cmelln(conversation_id: *const c_char,
                  lualias: *mut c_char,
                  return_code:    *mut c_int);

    pub fn cmesl(conversation_id: *const c_char,
                 sync_level: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmflus(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cminic(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmembs(conversation_id: *const c_char,
                  return_code:    *mut c_int);

    pub fn cmptr(conversation_id: *const c_char,
                 return_code:    *mut c_int);

    pub fn cmrts(conversation_id: *const c_char,
                 return_code:    *mut c_int);

    pub fn cmserr(conversation_id: *const c_char,
                 request_to_send_received: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsct(conversation_id: *const c_char,
                 conversation_type: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsed(conversation_id: *const c_char,
                 error_direction: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsf(conversation_id: *const c_char,
                fill_type: *const c_int,
                return_code:    *mut c_int);

    pub fn cmsld(conversation_id: *const c_char,
                 log_data: *const c_char,
                 log_data_length: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmslln(conversation_id: *const c_char,
                  lu_alias: *const c_char,
                  lu_alias_length: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmsmn(conversation_id: *const c_char,
                 mode_name: *const c_char,
                 mode_name_length: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsptr(conversation_id: *const c_char,
                  prepare_to_receive_type: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmsrt(conversation_id: *const c_char,
                 receive_type: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsrc(conversation_id: *const c_char,
                 return_control: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsst(conversation_id: *const c_char,
                 send_type: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmtrts(conversation_id: *const c_char,
                  request_to_send_received: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmspm(conversation_id: *const c_char,
                 processing_mode: *const c_int,
                 return_code:    *mut c_int);

    pub fn cmsltp(tp_name: *const c_char,
                  tp_name_length: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmrltp(tp_name: *const c_char,
                  tp_name_length: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmetpn(conversation_id: *const c_char,
                  tp_name: *mut c_char,
                  tp_name_length: *mut c_int,
                  return_code:    *mut c_int);

    pub fn xcmdsi(key: *const c_char,
                  sym_dest_name: *const c_char,
                  return_code:    *mut c_int);

    pub fn xcecst(conversation_id: *const c_char,
                  conversation_type: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmesui(conversation_id: *const c_char,
                  security_user_id: *mut c_char,
                  security_user_id_length: *mut c_int,
                  return_code:    *mut c_int);

    pub fn cmscsp(conversation_id: *const c_char,
                  security_password: *const c_char,
                  security_password_length: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmscst(conversation_id: *const c_char,
                  conversation_security_type: *const c_int,
                  return_code:    *mut c_int);

    pub fn cmscsu(conversation_id: *const c_char,
                  security_user_id: *const c_char,
                  security_user_id_length: *const c_int,
                  return_code:    *mut c_int);

}



